#include <p18cxxx.h>
#include <i2c.h>


/********************************************************************
*     Function Name:    EEByteWrite                                 *
*     Return Value:     error condition status                      *
*     Parameters:       EE memory device control, address and data  *
*                       bytes.                                      *
*     Description:      Write single data byte to I2C EE memory     *
*                       device. This routine can be used for any I2C*
*                       EE memory device, which only uses 1 byte of *
*                       address data as in the 24LC01B/02B/04B/08B. *
*                                                                   *  
********************************************************************/

unsigned char EEByteWrite_L( unsigned char control, unsigned int address, unsigned char data )
{
  unsigned char ByteValue ;
  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition

  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  else                            // start condition successful
  {
    if ( WriteI2C( control ) )    // write byte - R/W bit should be 0
    {
	  StopI2C(); 
      return ( -3 );              // set error for write collision
    }

    //IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition received
    { 

	  ByteValue = (address >> 8) & 0x00ff ;
      if ( WriteI2C(ByteValue) )  // write word address for EEPROM
      {
	    StopI2C(); 
        return ( -3 );            // set error for write collision
      }

      if ( WriteI2C( (char)(address & 0x00ff) ))  // write word address for EEPROM
      {
	    StopI2C(); 
        return ( -3 );            // set error for write collision
      }

      //IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition received
      { 
        if ( WriteI2C( data ) )   // data byte for EEPROM
        {
		  StopI2C(); 
          return ( -3 );          // set error for write collision
        }
      }
      else
      {
	    StopI2C(); 
        return ( -2 );            // return with Not Ack error condition   
      }
    }
    else
    {
	  StopI2C(); 
      return ( -2 );              // return with Not Ack error condition   
    }
  }

  //IdleI2C();                      // ensure module is idle  
  StopI2C();                      // send STOP condition
  while ( SSPCON2bits.PEN );      // wait until stop condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  return ( 0 );                   // return with no error
}


unsigned int EERandomRead_L( unsigned char control, unsigned int address )
{

  unsigned char ByteValue ;

  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition
  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  else
  {
    if ( WriteI2C( control ) )    // write 1 byte
    {
	  StopI2C(); 
      return ( -3 );              // return with write collision error
    }

    //IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   	// test for ACK condition, if received
    {
	  ByteValue = (address >> 8) & 0x00ff ;
      if ( WriteI2C( ByteValue ) )  		// write word address for EEPROM
      {
	    StopI2C(); 
        return ( -3 );            					// set error for write collision
      }

      if ( WriteI2C( (char)(address & 0x00ff) ))  // write word address for EEPROM
      {
	    StopI2C(); 
        return ( -3 );            // set error for write collision
      }



      //IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition, if received
      {
        RestartI2C();             // generate I2C bus restart condition
        while ( SSPCON2bits.RSEN );// wait until re-start condition is over 
        if ( PIR2bits.BCLIF )     // test for bus collision
        {
          return ( -1 );          // return with Bus Collision error 
        }
                          
        if ( WriteI2C( control+1 ) )// write 1 byte - R/W bit should be 1
        {
		  StopI2C(); 
          return ( -3 );          // return with write collision error
        }

        //IdleI2C();                // ensure module is idle
        if ( !SSPCON2bits.ACKSTAT )// test for ACK condition, if received
        {
          SSPCON2bits.RCEN = 1;       // enable master for 1 byte reception
          while ( SSPCON2bits.RCEN ); // check that receive sequence is over
          NotAckI2C();              // send ACK condition
          while ( SSPCON2bits.ACKEN ); // wait until ACK sequence is over 
          StopI2C();              // send STOP condition
          while ( SSPCON2bits.PEN ); // wait until stop condition is over 
          if ( PIR2bits.BCLIF )   // test for bus collision
          {
           return ( -1 );         // return with Bus Collision error 
          }
        }
        else
        {
		  StopI2C(); 
          return ( -2 );          // return with Not Ack error
        }

      }
      else
      {
	    StopI2C(); 
        return ( -2 );            // return with Not Ack error
      }
    }
    else
    {
	  StopI2C(); 
      return ( -2 );              // return with Not Ack error
    }
  }
  return ( (unsigned int) SSPBUF );     // return with data
}

